@@ -63,9 +63,7 @@ const char *getname(const uid_t uid)
 {
     static char number[9];
 
-    if (SNCHECK(snprintf(number, sizeof number, "%-8d", uid), sizeof number)) {
-        _EXIT(EXIT_FAILURE);
-    }
+    snprintf(number, sizeof number, "%-8d", uid);
     return number;
 }
 
@@ -73,9 +71,7 @@ const char *getgroup(const gid_t gid)
 {
     static char number[9];
 
-    if (SNCHECK(snprintf(number, sizeof number, "%-8d", gid), sizeof number)) {
-        _EXIT(EXIT_FAILURE);
-    }
+    snprintf(number, sizeof number, "%-8d", gid);
     return number;
 }
 #else
@@ -195,7 +191,7 @@ static void addfile(const char *name, const char *suffix)
 }
 
 /* listfile returns non-zero if the file is a directory */
-static int listfile(const FileInfo * const fi,  const char *name)
+static int listfile(const PureFileInfo * const fi,  const char *name)
 {
     int rval = 0;
     struct stat st;
@@ -266,8 +262,8 @@ static int listfile(const FileInfo * const fi,  const char *name)
         }
     }
     if (opt_l) {
-        strncpy(m, " ---------", (sizeof m) - 1U);
-        m[(sizeof m) - 1] = 0;
+        strncpy(m, " ---------", (sizeof m) - (size_t) 1U);
+        m[(sizeof m) - (size_t) 1U] = 0;
         switch (st.st_mode & S_IFMT) {
         case S_IFREG:
             m[0] = '-';
@@ -334,8 +330,9 @@ static int listfile(const FileInfo * const fi,  const char *name)
             }
 #ifdef WITH_LARGE_FILES
             if (SNCHECK(snprintf(alloca_nameline, sizeof_nameline,
-                                 "%s %4d %s %s %8llu %s %2d %s %s", 
-                                 m, st.st_nlink, getname(st.st_uid),
+                                 "%s %4u %s %s %8llu %s %2d %s %s", 
+                                 m, (unsigned int) st.st_nlink,
+				 getname(st.st_uid),
                                  getgroup(st.st_gid), 
                                  (unsigned long long) st.st_size,
                                  months[t->tm_mon],
@@ -343,8 +340,9 @@ static int listfile(const FileInfo * const fi,  const char *name)
                         sizeof_nameline))
 #else
             if (SNCHECK(snprintf(alloca_nameline, sizeof_nameline,
-                                 "%s %4d %s %s %8lu %s %2d %s %s", 
-                                 m, st.st_nlink, getname(st.st_uid),
+                                 "%s %4u %s %s %8lu %s %2d %s %s", 
+                                 m, (unsigned int) st.st_nlink,
+				 getname(st.st_uid),
                                  getgroup(st.st_gid), 
                                  (unsigned long) st.st_size,
                                  months[t->tm_mon],
@@ -378,7 +376,7 @@ static int listfile(const FileInfo * const fi,  const char *name)
                         suffix[0] = '*';
                     }
                 }
-                /* 2*MAXPATHLEN+gap should be enough, but be paranoid... */
+                /* 2 * MAXPATHLEN + gap should be enough, but be paranoid... */
                 if (SNCHECK
                     (snprintf(p, (sizeof_nameline) - strlen(alloca_nameline),
                               " -> %s", m), 
@@ -404,6 +402,9 @@ static void outputfiles(int f)
     unsigned int n;
     struct filename *p;
     struct filename *q;
+#ifdef WITH_RFC2640
+    char *c_buf; /* buffer with charset of client */
+#endif
 
     if (!head) {
         return;
@@ -472,7 +473,13 @@ static void outputfiles(int f)
                 pad[1] = '\n';
                 pad[2] = 0;
             }
+#ifdef WITH_RFC2640
+	    c_buf = charset_fs2client(q->line);
+            wrstr(f, c_buf);
+	    free(c_buf);
+#else
             wrstr(f, q->line);
+#endif
             wrstr(f, pad);
             q = q->right;
             free(tmp);
@@ -488,20 +495,22 @@ static void outputfiles(int f)
 /* functions to to sort for qsort() */
 static int cmp(const void * const a, const void * const b)
 {    
-    return strcmp(FI_NAME((const FileInfo *) a), FI_NAME((const FileInfo *) b));
+    return strcmp(FI_NAME((const PureFileInfo *) a),
+                  FI_NAME((const PureFileInfo *) b));
 }
 
 static int cmp_r(const void * const a, const void * const b)
 {    
-    return strcmp(FI_NAME((const FileInfo *) b), FI_NAME((const FileInfo *) a));
+    return strcmp(FI_NAME((const PureFileInfo *) b),
+                  FI_NAME((const PureFileInfo *) a));
 }
 
 static int cmp_t(const void * const a, const void * const b)
 {    
-    if (((const FileInfo *) a)->mtime < ((const FileInfo *) b)->mtime) {
+    if (((const PureFileInfo *) a)->mtime < ((const PureFileInfo *) b)->mtime) {
         return 1;
     }
-    if (((const FileInfo *) a)->mtime > ((const FileInfo *) b)->mtime) {
+    if (((const PureFileInfo *) a)->mtime > ((const PureFileInfo *) b)->mtime) {
         return -1;
     }
     return 0;
@@ -514,10 +523,10 @@ static int cmp_rt(const void * const a, const void * const b)
 
 static int cmp_S(const void * const a, const void * const b)
 {    
-    if (((const FileInfo *) a)->size < ((const FileInfo *) b)->size) {
+    if (((const PureFileInfo *) a)->size < ((const PureFileInfo *) b)->size) {
         return 1;
     }
-    if (((const FileInfo *) a)->size > ((const FileInfo *) b)->size) {
+    if (((const PureFileInfo *) a)->size > ((const PureFileInfo *) b)->size) {
         return -1;
     }
     return 0;
@@ -528,13 +537,13 @@ static int cmp_rS(const void * const a, const void * const b)
     return cmp_S(b, a);
 }
 
-static FileInfo *sreaddir(char **names_pnt)
+static PureFileInfo *sreaddir(char **names_pnt)
 {
     struct stat st;
     DIR *d;
     struct dirent *de;    
-    FileInfo *files_info;
-    FileInfo *file_info;
+    PureFileInfo *files_info;
+    PureFileInfo *file_info;
     size_t files_info_size;
     size_t files_info_counter = (size_t) 0U;
     char *names;
@@ -580,7 +589,7 @@ static FileInfo *sreaddir(char **names_pnt)
             names = new_names;
         }
         while ((files_info_counter + (size_t) 1U) >= files_info_size) {
-            FileInfo *new_files_info;
+            PureFileInfo *new_files_info;
             
             files_info_size += (CHUNK_SIZE / sizeof *files_info);
             if ((new_files_info = realloc(files_info, 
@@ -632,10 +641,10 @@ static FileInfo *sreaddir(char **names_pnt)
 /* have to change to the directory first (speed hack for -R) */
 static void listdir(unsigned int depth, int f, const char *name)
 {
-    FileInfo *dir;
+    PureFileInfo *dir;
     char *names;
-    FileInfo *s;
-    FileInfo *r;
+    PureFileInfo *s;
+    PureFileInfo *r;
     int d;
     
     if (depth >= max_ls_depth || matches >= max_ls_files) {
@@ -777,7 +786,7 @@ void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
         }
     }
     if (on_ctrl_conn == 0) {
-	opendata();
+    opendata();
         if ((c = xferfd) == -1) {
             return;
         }
